Package: resourcecode
Title: Access to the 'RESOURCECODE' Hindcast Database
Version: 0.5.4
Authors@R: 
    person("Nicolas", "Raillard", , "nicolas.raillard@ifremer.fr", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-3385-5104"))
Description: Utility functions to download data from the 'RESOURCECODE'
    hindcast database of sea-states, time series of sea-state parameters
    and time series of 1D and 2D wave spectra.  See
    <https://resourcecode.ifremer.fr> for more details about the available
    data.  Also provides facilities to plot and analyse downloaded data,
    such as computing the sea-state parameters from both the 1D and 2D
    surface elevation variance spectral density.
License: GPL (>= 3)
URL: https://github.com/Resourcecode-project/r-resourcecode,
        https://resourcecode-project.r-universe.dev/resourcecode
BugReports: https://github.com/Resourcecode-project/r-resourcecode/issues
Depends: R (>= 4.1)
Imports: abind, geosphere, ggplot2, grid, gridtext, httr2, lubridate,
        ncdf4, patchwork, pracma, Rcpp, resourcecodedata, rlang, sf,
        stats, tibble, tidyr
Suggests: knitr, mockery, rmarkdown, testthat, vcr
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
Language: en-US
LazyData: true
NeedsCompilation: yes
Packaged: 2026-05-06 20:06:56 UTC; nraillar
Author: Nicolas Raillard [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3385-5104>)
Maintainer: Nicolas Raillard <nicolas.raillard@ifremer.fr>
Repository: CRAN
Date/Publication: 2026-05-07 09:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-07 11:42:45 UTC; unix
Archs: resourcecode.so.dSYM
