# Generated by roxygen2: do not edit by hand

export("%nin%")
export(closest_point_field)
export(closest_point_spec)
export(compute_orbital_speeds)
export(compute_sea_state_1d_spectrum)
export(compute_sea_state_2d_spectrum)
export(convert_spectrum_2d1d)
export(cut_directions)
export(cut_seasons)
export(dispersion)
export(fastTrapz)
export(fractional_day_of_year)
export(get_1d_spectrum)
export(get_2d_spectrum)
export(get_parameters)
export(jonswap)
export(mean_direction)
export(metconv2zmcomp)
export(plot_1d_specta)
export(plot_2d_specta)
export(rscd_mapplot)
export(weather_windows)
export(zmcomp2metconv)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_polar)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,expansion)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_distiller)
importFrom(ggplot2,scale_fill_distiller)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_linedraw)
importFrom(ggplot2,theme_void)
importFrom(rlang,.data)
useDynLib(resourcecode, .registration = TRUE)
