#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom ggplot2 ggplot aes geom_path coord_sf theme_void geom_polygon scale_fill_distiller
#' @importFrom ggplot2 scale_color_distiller theme margin element_rect element_blank geom_rect
#' @importFrom Rcpp sourceCpp evalCpp
#' @importFrom rlang .data
#' @useDynLib resourcecode, .registration = TRUE
## usethis namespace: end
NULL
