# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Fast implementation of pracma::trapz from the Armadillo C++ library
#'
#' Compute the area of a multivariate function with (matrix) values Y at the points x.
#'
#' @param x x-coordinates of points on the x-axis (vector)
#' @param Y y-coordinates of function values (matrix)
#' @param dim an integer giving the subscripts which the function will be applied over. 1 indicates rows, 2 indicates columns
#' @return a vector with one dimension less than Y
#' @export
#' @examples
#' x = 1:10
#' Y = sin(pi/10*matrix(1:10,ncol=10,nrow=10))
#' fastTrapz(x*pi/10,Y,2)
fastTrapz <- function(x, Y, dim = 1L) {
    .Call(`_resourcecode_fastTrapz`, x, Y, dim)
}

ww_calc_cpp <- function(times, winlen_hours, allow_overlap, tstep_secs) {
    .Call(`_resourcecode_ww_calc_cpp`, times, winlen_hours, allow_overlap, tstep_secs)
}

